package init;

import java.io.*;
import java.nio.channels.FileChannel;
import java.util.*;

/**
 * Windows用 AsakusaFWのEclipseプロジェクトで使っているjarファイルを Hadoopのディレクトリーにコピーする。
 *
 * @author <a
 *         href="http://www.ne.jp/asahi/hishidama/home/tech/asakusafw/install.html"
 *         >hishidama</a>
 * @since 2011.07.31
 */
public class HadoopCDH3 {
	public File getHadoopDir() {
		// Hadoopのディレクトリー
		return new File("C:/cygwin/usr/local/hadoop-0.20.2-cdh3");
	}

	public File getWorkspaceDir() {
		// Eclipseのワークスペースのプロジェクトディレクトリーを指しているはず
		return new File(System.getProperty("user.dir"));
	}

	public File getM2RepoDir() {
		// Mavenのリポジトリー
		return new File(System.getProperty("user.home"), ".m2/repository");
	}

	void run() throws IOException {
		deleteHadoopCore();

		List<String> list = getJarList();
		File dst = getHadoopDir();
		copy(list, dst);
	}

	List<String> getJarList() throws IOException {
		List<String> list = new ArrayList<String>();

		File dir = getWorkspaceDir();
		File f = new File(dir, ".classpath");
		BufferedReader br = new BufferedReader(new InputStreamReader(
				new FileInputStream(f)));
		try {
			for (;;) {
				String line = br.readLine();
				if (line == null)
					break;
				int n = line.indexOf(" path=\"M2_REPO/");
				if (n > 0) {
					n += 15;
					int m = line.indexOf('"', n);
					list.add(line.substring(n, m));
				}
			}
		} finally {
			br.close();
		}

		return list;
	}

	void copy(List<String> list, File dstDir) throws IOException {
		File srcDir = getM2RepoDir();
		for (String s : list) {
			File src = new File(srcDir, s);
			File dst = getDstFile(dstDir, src);
			copy(src, dst);
		}
	}

	// hadoop0.20実行時には「hadoop-*-core.jar」が使われるが
	// CDH3は「hadoop-core-0.20.2-cdh3u0.jar」だからマッチしないので、マッチする名前に変更する
	File getDstFile(File dir, File f) {
		String name = f.getName();
		if (name.contains("hadoop-core")) {
			return new File(dir, name + "-core.jar");
		} else {
			return new File(dir, "lib/" + name);
		}
	}

	// 元来の「hadoop-*-core.jar」を削除する
	void deleteHadoopCore() {
		File dir = getHadoopDir();
		for (File f : dir.listFiles()) {
			String name = f.getName();
			if (name.startsWith("hadoop-") && name.endsWith("-core.jar")) {
				System.out.println("delete\t" + f);
				f.delete();
			}
		}
	}

	void copy(File s, File d) throws IOException {
		System.out.println("copy\t" + s + "\t" + d);

		// ファイルコピーはJDK1.7なら簡単なのにー！
		// import java.nio.file.Files;
		// import static java.nio.file.StandardCopyOption.*;
		// Files.copy(s.toPath(), d.toPath(), COPY_ATTRIBUTES,REPLACE_EXISTING);

		FileChannel sc = null, dc = null;
		try {
			sc = new FileInputStream(s).getChannel();
			dc = new FileOutputStream(d).getChannel();
			dc.transferFrom(sc, 0, sc.size());
		} finally {
			if (dc != null)
				try {
					dc.close();
				} catch (IOException e) {
				}
			if (sc != null)
				try {
					sc.close();
				} catch (IOException e) {
				}
		}
	}

	public static void main(String[] args) throws IOException {
		// System.getProperties().list(System.out);
		new HadoopCDH3().run();
	}
}
